"""Login interface"""
from pydantic import BaseModel


class ILoginRequest(BaseModel):
    """Login request model"""

    username: str
    password: str


class ILoginResponse(BaseModel):
    """Login response model"""

    auth_token: str



"""Clicker module"""
from typing import List

from httpx import get
from httpx import put

from pilk_clicker.interfaces.auth import ITokenRequest
from pilk_clicker.interfaces.clicker import IClickerDetailResponse
from pilk_clicker.interfaces.clicker import IClickerSaveRequest
from pilk_clicker.interfaces.clicker import IClickerSaveResponse
from pilk_clicker.interfaces.clicker import ITopListResponse
from pilk_clicker.urls import ApiUrls


class Clicker:
    @staticmethod
    def clicker_detail(credentials: ITokenRequest) -> IClickerDetailResponse:
        """Get clicker detail

        :param credentials: Token request model
        :return: Clicker detail response model
        """
        response = get(
            ApiUrls.URL_CLICKER,
            headers={"Authorization": f"Token {credentials.authorization}"},
        )
        response.raise_for_status()
        return IClickerDetailResponse(**response.json())

    @staticmethod
    def top_list(credentials: ITokenRequest) -> List[ITopListResponse]:
        """Get top list

        :param credentials: Token request model
        :return: Top list response model
        """
        response = get(
            ApiUrls.URL_TOP_LIST,
            headers={"Authorization": f"Token {credentials.authorization}"},
        )
        response.raise_for_status()
        return [ITopListResponse(**item) for item in response.json()]

    @staticmethod
    def save_clicker(
        data: IClickerSaveRequest, credentials: ITokenRequest
    ) -> IClickerSaveResponse:
        """Save clicker

        :param data: Clicker save request model
        :param credentials: Token request model
        :return: Clicker save response model
        """
        response = put(
            ApiUrls.URL_SAVE_CLICKER,
            json=data.model_dump(),
            headers={"Authorization": f"Token {credentials.authorization}"},
        )
        response.raise_for_status()
        return IClickerSaveResponse(**response.json())



"""Interfaces for the Pilk Clicker API."""
from pilk_clicker.interfaces import auth
from pilk_clicker.interfaces import clicker
from pilk_clicker.interfaces import shop


__all__ = [
    "auth",
    "clicker",
    "shop",
]



"""Shop module"""
from typing import List

from httpx import get
from httpx import put

from pilk_clicker.interfaces.auth import ITokenRequest
from pilk_clicker.interfaces.shop import ISaveItemRequest
from pilk_clicker.interfaces.shop import ISaveItemResponse
from pilk_clicker.interfaces.shop import IShopUserResponse
from pilk_clicker.urls import ApiUrls


class Shop:
    """Shop class"""

    @staticmethod
    def shop_user(credentials: ITokenRequest) -> List[IShopUserResponse]:
        """Get shop user

        :param credentials: Token request model
        :return: Shop user response model
        """
        response = get(
            ApiUrls.URL_SHOP_USER_DETAIL,
            headers={"Authorization": f"Token {credentials.authorization}"},
        )
        response.raise_for_status()
        return [IShopUserResponse(**item) for item in response.json()]

    @staticmethod
    def save_item(
        data: ISaveItemRequest, credentials: ITokenRequest
    ) -> ISaveItemResponse:
        """Save item

        :param data: Save item request model
        :param credentials: Token request model
        :return: Save item response model
        """
        response = put(
            ApiUrls.URL_SAVE_ITEM,
            json=data.model_dump(),
            headers={"Authorization": f"Token {credentials.authorization}"},
        )
        response.raise_for_status()
        return ISaveItemResponse(**response.json())



"""Modules for pilk_clicker."""
from pilk_clicker.modules.auth import Auth
from pilk_clicker.modules.clicker import Clicker
from pilk_clicker.modules.shop import Shop


__all__ = ["Clicker", "Auth", "Shop"]



"""Interfaces for clicker module."""
from .clicker import IClickerDetailResponse
from .clicker import IClickerSaveRequest
from .clicker import IClickerSaveResponse
from .clicker import ITopListResponse


__all__ = [
    "IClickerDetailResponse",
    "IClickerSaveRequest",
    "IClickerSaveResponse",
    "ITopListResponse",
]



"""Clicker interfaces"""
from pydantic import BaseModel


class IClickerDetailResponse(BaseModel):
    """Clicker detail response model"""

    username: str
    arcoin_amount: int
    arcoins_per_seconds: int
    arcoins_per_click: int


class IClickerSaveRequest(BaseModel):
    """Clicker save request model"""

    arcoin_amount: int
    arcoins_per_seconds: int
    arcoins_per_click: int


class IClickerSaveResponse(BaseModel):
    """Clicker save response model"""

    arcoin_amount: int
    arcoins_per_seconds: int
    arcoins_per_click: int


class ITopListResponse(BaseModel):
    """Top list model"""

    username: str
    arcoin_amount: int



"""Logup interface"""
from pydantic import BaseModel


class ILogupRequest(BaseModel):
    """Logup request model"""

    username: str
    password: str
    email: str


class ILogupResponse(BaseModel):
    """Logup response model"""

    id: int
    username: str
    email: str



"""Shop interfaces"""
from pydantic import BaseModel


class ISaveItemRequest(BaseModel):
    """Save item request model"""

    id: int
    amount: int


class ISaveItemResponse(BaseModel):
    """Save item response model"""

    id: int
    name: str
    amount: int
    price: int
    base_price: int
    arcoins_per_seconds: float
    arcoins_per_click: int
    image: str


class IShopUserResponse(BaseModel):
    """Shop user response model"""

    id: int
    name: str
    amount: int
    price: int
    base_price: int
    arcoins_per_seconds: float
    arcoins_per_click: int
    image: str



"""Auth module"""
from httpx import post

from pilk_clicker.interfaces.auth import ILoginRequest
from pilk_clicker.interfaces.auth import ILoginResponse
from pilk_clicker.interfaces.auth import ILogupRequest
from pilk_clicker.interfaces.auth import ILogupResponse
from pilk_clicker.interfaces.auth import ITokenRequest
from pilk_clicker.urls import ApiUrls


class Auth:
    """Auth module"""

    @staticmethod
    def login(credentials: ILoginRequest) -> ILoginResponse:
        """Login to the api

        :param credentials: Login request model
        :return: Login response model
        """
        response = post(ApiUrls.URL_AUTH_LOGIN, json=credentials.model_dump())
        response.raise_for_status()
        return ILoginResponse(**response.json())

    @staticmethod
    def logout(credentials: ITokenRequest) -> None:
        """Logout from the api

        :param credentials: Token request model
        """
        response = post(
            ApiUrls.URL_AUTH_LOGOUT,
            headers={"Authorization": f"Token {credentials.authorization}"},
        )
        response.raise_for_status()

    @staticmethod
    def logup(credentials: ILogupRequest) -> ILogupResponse:
        """Logup to the api

        :param credentials: Logup request model
        :return: Logup response model
        """
        response = post(ApiUrls.URL_USERS, json=credentials.model_dump())
        response.raise_for_status()
        return ILogupResponse(**response.json())



"""Interfaces for auth module."""
from .login import ILoginRequest
from .login import ILoginResponse
from .logup import ILogupRequest
from .logup import ILogupResponse
from .token_request import ITokenRequest


__all__ = [
    "ILogupRequest",
    "ILogupResponse",
    "ILoginRequest",
    "ILoginResponse",
    "ITokenRequest",
]



"""Import all modules from pilk_clicker."""
from pilk_clicker import interfaces
from pilk_clicker.modules import Auth
from pilk_clicker.modules import Clicker
from pilk_clicker.modules import Shop


__all__ = ["Clicker", "Auth", "Shop", "interfaces"]



"""Interfaces for shop module."""
from .shop import ISaveItemRequest
from .shop import ISaveItemResponse
from .shop import IShopUserResponse


__all__ = [
    "ISaveItemRequest",
    "ISaveItemResponse",
    "IShopUserResponse",
]



"""Urls for pilk_clicker api"""
from typing import Final


class ApiUrls:
    """Api urls"""

    _URL: Final = "http://pilk-clicker.ru/api"

    _URL_AUTH: Final = f"{_URL}/auth"
    URL_AUTH_LOGIN: Final = f"{_URL_AUTH}/token/login/"
    URL_AUTH_LOGOUT: Final = f"{_URL_AUTH}/token/logout/"

    URL_USERS: Final = f"{_URL_AUTH}/users/"

    URL_CLICKER: Final = f"{_URL}/clicker/"
    URL_SAVE_CLICKER: Final = f"{_URL}/save_clicker/"
    URL_TOP_LIST: Final = f"{_URL}/top_list/"

    URL_SHOP_USER_DETAIL: Final = f"{_URL}/shop_user_detail/"
    URL_SAVE_ITEM: Final = f"{_URL}/save_item/"
